/*
 * Copyright (c) 2016-2017 Satlab Aps <satlab@satlab.com>
 * All rights reserved. Do not distribute without permission.
 */

#ifndef _SL_SRS4_CLIENT_PROTO_H_
#define _SL_SRS4_CLIENT_PROTO_H_

#include <stdint.h>

/* Default CSP address */
#define SL_SRS4_DEFAULT_ADDRESS		24

/** CSP ports */
#define SL_SRS4_PORT_BOOTLOADER		25

/** Error codes */
#define SL_SRS4_ERR_NONE		0
#define SL_SRS4_ERR_INVAL		1
#define SL_SRS4_ERR_IO			2
#define SL_SRS4_ERR_NOMEM		3
#define SL_SRS4_ERR_BUSY		4

/* Generic packet types */
typedef struct {
	uint8_t type;
} __attribute__ ((packed)) sl_srs4_req_t;

typedef struct {
	uint8_t type;
	uint8_t error;
} __attribute__ ((packed)) sl_srs4_rep_t;

/*
 * SL_SRS4_PORT_BOOTLOADER
 */

/* Types */
#define SL_SRS4_BOOTLOADER_SET_REQ		1
#define SL_SRS4_BOOTLOADER_SET_REP		2
#define SL_SRS4_BOOTLOADER_FLASH_REQ		5
#define SL_SRS4_BOOTLOADER_FLASH_REP		6
#define SL_SRS4_BOOTLOADER_ERASE_REQ		7
#define SL_SRS4_BOOTLOADER_ERASE_REP		8
#define SL_SRS4_BOOTLOADER_CHECKSUM_REQ		9
#define SL_SRS4_BOOTLOADER_CHECKSUM_REP		10

/* Keys */
#define SL_SRS4_BOOTLOADER_SET_KEY		0x4f479511
#define SL_SRS4_BOOTLOADER_FLASH_KEY		0x1f57ef1f
#define SL_SRS4_BOOTLOADER_ERASE_KEY		0xcd6d7fb8

typedef struct {
	/* Must be SL_SRS4_BOOTLOADER_SET_REQ */
	uint8_t type;
	/* Flags, currently unused - set to zero */
	uint8_t flags;
	/* Number of boots to boot alternate image, 0 = boot factory */
	uint8_t boots;
	/* Must be SL_SRS4_BOOTLOADER_SET_KEY */
	uint32_t key;
} __attribute__ ((packed)) sl_srs4_bootloader_set_req_t;

typedef struct {
	/* Must be SL_SRS4_BOOTLOADER_SET_REP */
	uint8_t type;
	/* Error code */
	uint8_t error;
} __attribute__ ((packed)) sl_srs4_bootloader_set_rep_t;

#define SL_SRS4_BOOTLOADER_FILENAME_MAX	40
typedef struct {
	/* Must be SL_SRS4_BOOTLOADER_FLASH_REQ */
	uint8_t type;
	/* Flags, currently unused - set to zero */
	uint8_t flags;
	/* Must be SL_SRS4_BOOTLOADER_FLASH_KEY */
	uint32_t key;
	/* Filename to flash to the alternate partition */
	uint8_t filename[SL_SRS4_BOOTLOADER_FILENAME_MAX];
} __attribute__ ((packed)) sl_srs4_bootloader_flash_req_t;

typedef struct {
	/* Must be SL_SRS4_BOOTLOADER_FLASH_REP */
	uint8_t type;
	/* Error code */
	uint8_t error;
	/* CRC32C of flash image */
	uint32_t checksum;
} __attribute__ ((packed)) sl_srs4_bootloader_flash_rep_t;

typedef struct {
	/* Must be SL_SRS4_BOOTLOADER_ERASE_REQ */
	uint8_t type;
	/* Flags, currently unused - set to zero */
	uint8_t flags;
	/* Must be SL_SRS4_BOOTLOADER_ERASE_KEY */
	uint32_t key;
} __attribute__ ((packed)) sl_srs4_bootloader_erase_req_t;

typedef struct {
	/* Must be SL_SRS4_BOOTLOADER_ERASE_REP */
	uint8_t type;
	/* Error code */
	uint8_t error;
} __attribute__ ((packed)) sl_srs4_bootloader_erase_rep_t;

typedef struct {
	/* Must be SL_SRS4_BOOTLOADER_CHECKSUM_REQ */
	uint8_t type;
	/* Flags, currently unused - set to zero */
	uint8_t flags;
	/* Number of bytes to checksum from beginning of alternate partition */
	uint32_t size;
} __attribute__ ((packed)) sl_srs4_bootloader_checksum_req_t;

typedef struct {
	/* Must be SL_SRS4_BOOTLOADER_CHECKSUM_REP */
	uint8_t type;
	/* Error code */
	uint8_t error;
	/* CRC32C of flash image */
	uint32_t checksum;
} __attribute__ ((packed)) sl_srs4_bootloader_checksum_rep_t;

#endif /* _SL_SRS4_CLIENT_PROTO_H_ */
